/*
 * Bob Manchek's instrumented malloc library
 * hacked over for various reasons by Keith Moore
 *
 * XXX Now that HeNCE links in libpvm3.a, we now have two imalloc 
 * packages in the same executable.  This needs to be resolved.
 */

/*
 *  imalloc.c
 *
 *  Instrumented malloc filter.
 *
 *  7 Jun 1991	Robert Manchek	manchek@CS.UTK.EDU.
 *  6 Sep 1991	added static glob and check features.
 *  17 Dec 1991 added x_realloc, cleaned up.
 *  8 Sep 1992	added total byte count.
 *  30 Oct 1992 added object type tag
 *
 *  To use, add something like the following to your code:
 *	#define malloc(n) x_malloc(n)
 *	#define realloc(p,n) x_realloc(p,n)
 *	#define free(p) x_free(p)
 *  recompile, and link.
 *
 *  Facilities:
 *
 *  $ All errors written to fd 2 (not using stdio).
 *
 *  $ Configurable to die on error for debugger.
 *	Define DIEONERROR as 0 or 1.
 *
 *  $ Configurable to use statically allocated space for recordkeeping.
 *	This makes imalloc more immune to bad heap space trashing.
 *	Define STATICGLOBS as nonzero.
 *
 *  $ X_malloc():
 *	Error on [adjustable] unreasonble length or malloc() failed.
 *	Hashes descriptors of all chunks for x_free() checking.
 *	Numbers requests to indicate order received.
 *	Writes [adjustable] pseudo-random head and tail pads to
 *	    aid in checking overrun of chunk.
 *
 *  $ X_free(ptr):
 *	Error if chunk not x_malloc()d.
 *	Checks head and tail pads.
 *	[Optionally] zeros chunk to aid in detecting late use.
 *	Define ZEROONFREE as 0 or 1.
 *
 *  $ X_dump(how):
 *	Callable from user program.
 *	Dumps the hash table to see what chunks are active.
 *	If how is 1, checks each chunk's head and tail pads.
 *
 */

#include "Malloc.h"

#ifdef DEBUG_MALLOC
#if 0
#ifdef	SYSVSTR
#include <string.h>
#else
#include <strings.h>
#endif
#endif

#ifndef DIEONERROR
#define DIEONERROR  1
#endif
#ifndef ZEROONFREE
#define ZEROONFREE  1
#endif
#ifndef STATICGLOBS
#define STATICGLOBS 0
#endif
#ifndef DUMPHDR
#define DUMPHDR "x_dump()"
#endif

#define NEXTRN(x)   (x = (x) + (x) + (((x ^ (x >> 3)) & 0x2000) ? 1 : 0))
#define HASH(p)	    (((int)(p) ^ ((int)(p) >> 8) ^ ((int)(p) >> 16) ^ ((int)(p) >> 24)) & 0xff)

/* write a null-term string */


#define SWRITE(fd,s) write((fd),(s),strlen(s))
#if 0
#define SWRITE(fd,s) log_error(s)
#endif
/* this describes a chunk of memory */

struct glob {
    struct glob *next;	/* next glob in hash bucket */
    char *base;		/* baseaddr of user chunk */
    int len;		/* len of user chunk */
    int id;		/* chunk id */
    int lop;		/* nbytes head padding */
    int hip;		/* nbytes tail padding */
    int rst;		/* starting random state */
    int flg;
    char *tag;		/* tag */
    int line;		/* line # */
};

#define OBALLOC	    1	/* ob was just alloced - cleared by dump() */
#define OBREALLOC   2	/* ob was just realloced - cleared by dump() */

/* default values */

static int debfd = 2;		    /* debug file descriptor */
static int lengthlimit = 1048576;   /* max length malloc allowed */
static int lopad = 16;		    /* chunk head pad */
static int hipad = 16;		    /* chunk tail pad */

/* globals */

static int totlnbyts = 0;	    /* total bytes allocated */
static int rnstate = 1;		    /* random sequence gen. */
static struct glob *hashtbl[256];   /* chunk hash table */
static char msbuf[256];		    /* error message buffer */
static int firsttime = 1;
static int globid = 0;		    /* chunk id counter */
#if STATICGLOBS > 0
static struct glob globheap[STATICGLOBS];
static struct glob *globfl = 0;
static int globavail = STATICGLOBS;
#endif


/*  x_choke()
*
*   A place to fail.
*/

x_choke()
{
#if DIEONERROR
    char *p = 0;

    *p = 1;
#endif
}


/*  x_malloc()
*
*   Allocate a buffer of given length.
*/

void *
x_malloc (len, tag, line)
int len;    /* number of bytes */
char *tag;
int line;
{
    char *ptr;
    struct glob *ob;	/* hash tbl entry */
    struct glob **he;
    int i;		/* gp */

    if (firsttime) {
	firsttime = 0;
	bzero((char*)hashtbl, sizeof(hashtbl));

#if STATICGLOBS > 0
    /* initialize the glob freelist */
	ob = 0;
	for (i = STATICGLOBS-1; i >= 0; i--) {
	    globheap[i].next = ob;
	    ob = &globheap[i];
	}
	globfl = ob;
#endif
    }

    /* check req length */

    if (len < 1 || len > lengthlimit) {
	(void)sprintf(msbuf, "x_malloc: bogus len=%d\n", len);
	(void)SWRITE(debfd, msbuf);
	x_choke();
	return (char*)0;
    }

    /* do actual malloc */

    if (!(ptr = (char*)malloc(len + lopad + hipad))) {
	(void)sprintf(msbuf, "x_malloc: malloc failed len=%d\n", len);
	(void)SWRITE(debfd, msbuf);
	x_choke();
	return (char*)0;
    }

    /* get descriptor */

#if STATICGLOBS > 0
    if (ob = globfl) {
	globfl = globfl->next;
	globavail--;

    } else {
	(void)sprintf(msbuf, "x_malloc: glob allocate failed (max %d)\n",
	    STATICGLOBS);
	(void)SWRITE(debfd, msbuf);
	x_choke();
	return (char*)0;
    }
#else
    if (!(ob = (struct glob*)malloc(sizeof(struct glob)))) {
	(void)sprintf(msbuf, "x_malloc: malloc failed for glob\n");
	(void)SWRITE(debfd, msbuf);
	x_choke();
	return (char*)0;
    }
#endif

    /* enter descriptor, write head and tail pads */

    ob->flg = OBALLOC;
    ob->id = ++globid;
    ob->tag = tag;
    ob->line = line;
    ob->len = len;
    ob->lop = lopad;
    ob->hip = hipad;
    ob->rst = rnstate;
    for (i = lopad; i-- > 0; *ptr++ = NEXTRN(rnstate));
    ob->base = ptr;
    he = &hashtbl[HASH(ptr)];
    for (i = hipad, ptr += len; i-- > 0; *ptr++ = NEXTRN(rnstate));
    ob->next = *he;
    *he = ob;
    totlnbyts += len;
    return ob->base;
}


/*  x_free()
*
*   Give up a previously allocated buffer.
*/

x_free(loc)
char *loc;	    /* ptr to buffer */
{
    struct glob *ob;	/* freeing this object */
    struct glob *preob; /* object before in chain */
    int rs;		/* reproduced random seqn */
    char *ptr = loc;	/* gp */
    struct glob **he;	/* hash tbl entry */
    int i;		/* gp */

    /* sanity check */

    if (firsttime) {
	char *s = "x_free: called before x_malloc?\n";
	SWRITE(debfd, s);
	x_choke();
	return;
    }

    /* delete from hash tbl */

    he = &hashtbl[HASH(loc)];
    for (preob = 0, ob = *he; ob && ob->base != loc; preob = ob, ob = ob->next);
    if (!ob) {
	(void)sprintf(msbuf, "x_free: bogus loc=0x%x\n", loc);
	(void)SWRITE(debfd, msbuf);
	x_choke();
	return;
    }
    rs = ob->rst;

    /* check head and tail pads */

    for (i = ob->lop, ptr -= i; i > 0; i--)
	if ((0xff & (int)(*ptr++)) != (0xff & NEXTRN(rs))) {
	    (void)sprintf(msbuf, "x_free: scribbled in 0x%x[%d](%s/%d)\n",
			  loc, -i, ob->tag, ob->line);
	    (void)SWRITE(debfd, msbuf);
	    x_choke();
	}
    for (i = ob->hip, ptr += ob->len; i > 0; i--)
	if ((0xff & (int)(*ptr++)) != (0xff & NEXTRN(rs))) {
	    (void)sprintf(msbuf, "x_free: scribbled in 0x%x[%d+%d](%s/%d)\n",
		loc, ob->len, ob->hip - i, ob->tag, ob->line);
	    (void)SWRITE(debfd, msbuf);
	    x_choke();
	}

    /* do actual free */

#if ZEROONFREE
    bzero(loc - ob->lop, ob->len + ob->lop + ob->hip);
#endif
    free(loc - ob->lop);
    totlnbyts -= ob->len;

    /* reclaim descriptor */

    if (preob)
	preob->next = ob->next;
    else
	*he = ob->next;
#if STATICGLOBS > 0
    ob->next = globfl;
    globfl = ob;
    globavail++;
#else
    free((char*)ob);
#endif
}


/*  x_realloc()
*
*   Resize a previously allocated buffer (possibly relocate as well)
*   and leave the contents unchanged up to the minimum of the old
*   and new lengths.
*/

void *
x_realloc(loc, len)
char *loc;		/* old buffer */
int len;		/* length of new buffer */
{
    struct glob *ob;	/* freeing this object */
    struct glob *preob; /* object before in chain */
    int rs;		/* reproduced random seqn */
    char *ptr = loc;	/* gp */
    struct glob **he;	/* hash tbl entry */
    int i;		/* gp */

    /* sanity check */

    if (firsttime) {
	char *s = "x_realloc: called before x_malloc?\n";
	SWRITE(debfd, s);
	x_choke();
	return (char*)0;
    }

    /* check req length */

    if (len < 1 || len > lengthlimit) {
	(void)sprintf(msbuf, "x_realloc: bogus len=%d\n", len);
	(void)SWRITE(debfd, msbuf);
	x_choke();
	return (char*)0;
    }

    /* delete from hash tbl */

    he = &hashtbl[HASH(loc)];
    for (preob = 0, ob = *he; ob && ob->base != loc; preob = ob, ob = ob->next);
    if (!ob) {
	(void)sprintf(msbuf, "x_realloc: bogus loc=0x%x\n", loc);
	(void)SWRITE(debfd, msbuf);
	x_choke();
	return (char*)0;
    }
    rs = ob->rst;

    /* check head and tail pads */

    for (i = ob->lop, ptr -= i; i > 0; i--)
	if ((0xff & (int)(*ptr++)) != (0xff & NEXTRN(rs))) {
	    (void)sprintf(msbuf, "x_realloc: scribbled in 0x%x[%d]\n", loc, -i);
	    (void)SWRITE(debfd, msbuf);
	    x_choke();
	}
    for (i = ob->hip, ptr += ob->len; i > 0; i--)
	if ((0xff & (int)(*ptr++)) != (0xff & NEXTRN(rs))) {
	    (void)sprintf(msbuf, "x_realloc: scribbled in 0x%x[%d+%d]\n",
		loc, ob->len, ob->hip - i);
	    (void)SWRITE(debfd, msbuf);
	    x_choke();
	}

    /* remove descriptor */

    if (preob)
	preob->next = ob->next;
    else
	*he = ob->next;

    /* realloc */

    if (!(ptr = (char*)realloc(loc - ob->lop, len + lopad + hipad))) {
	(void)sprintf(msbuf, "x_realloc: malloc failed len=%d\n", len);
	(void)SWRITE(debfd, msbuf);
	x_choke();
	return (char*)0;
    }

    /* rewrite descriptor, write head and tail pads */

    totlnbyts += len - ob->len;
    ob->flg = OBREALLOC;
    ob->id = ++globid;	/* XXX ? */
    ob->len = len;
    ob->lop = lopad;
    ob->hip = hipad;
    ob->rst = rnstate;
    for (i = lopad; i-- > 0; *ptr++ = NEXTRN(rnstate));
    ob->base = ptr;
    he = &hashtbl[HASH(ptr)];
    for (i = hipad, ptr += len; i-- > 0; *ptr++ = NEXTRN(rnstate));
    ob->next = *he;
    *he = ob;
    return ob->base;
}


/*  ascdump()
*
*   Convert byte string to printable characters.
*/

static
ascdump(o, p, e)
char **o;	    /* addr of pointer to output space (passed back) */
char *p;	    /* string to convert */
int e;	    /* length of string */
{
    char *r = *o;
    char c;

    while (e-- > 0) {
	c = 0x7f & *p;
	if (c < ' ' || c == 0x7f) {
	    c = (c + '@') & 0x7f;
	    *r++ = '^';
	} else
	    *r++ = ' ';
	*r++ = c;
	p++;
    }
    *r++ = '\n';
    *r = 0;
    *o = r;
}


/*  x_dump()
*
*   Dump the table of contents of allocated buffers.
*/

x_dump(how)
int how;		/* != 0 also do sanity checking */
{
    int ht;		/* hash table index */
    struct glob *ob;
    char *r;
    int i;
    int rs;		/* reproduced random seqn */
    char *ptr;		/* gp */
    int err;
    char *tag;

#if STATICGLOBS > 0
    sprintf(msbuf, "%s %d bytes total/%d globs free\n",
	    DUMPHDR, totlnbyts, globavail);
    (void)SWRITE(debfd, msbuf);
#else
    sprintf(msbuf, "%s %d bytes total\n", DUMPHDR, totlnbyts);
    (void)SWRITE(debfd, msbuf);
#endif
    for (ht = 0; ht < 256; ht++) {
	for (ob = hashtbl[ht]; ob; ob = ob->next) {
	    rs = ob->rst;
	    ptr = ob->base;
	    err = 0;
	    if (how) {
		for (i = ob->lop, ptr -= i; i > 0; i--)
		    if ((0xff & (int)(*ptr++)) != (0xff & NEXTRN(rs))) {
			(void)sprintf(msbuf,
			    "%5d 0x%08x[%d]: scribbled in [%d]\n",
			    ob->id, ob->base, ob->len, -i);
			(void)SWRITE(debfd, msbuf);
			err++;
		    }
		for (i = ob->hip, ptr += ob->len; i > 0; i--)
		    if ((0xff & (int)(*ptr++)) != (0xff & NEXTRN(rs))) {
			(void)sprintf(msbuf,
			    "%5d 0x%08x[%d]: scribbled in [%d+%d]\n",
			    ob->id, ob->base, ob->len, ob->len, ob->hip - i);
			(void)SWRITE(debfd, msbuf);
			err++;
		    }
	    }
	    if (!err) {
		sprintf(msbuf, "%5d%c%s/%d 0x%08x[%4d]",
		    ob->id,
		    (ob->flg & OBALLOC ? '*' :
		     (ob->flg & OBREALLOC ? '+' : ' ')),
		    ob->tag, ob->line,
		    ob->base, ob->len);
		r = msbuf + strlen(msbuf);
		*r++ = ' ';
		if ((i = ob->len) > 24)
		    i = 24;
		ascdump(&r, ob->base, i);
		(void)SWRITE(debfd, msbuf);
	    }
	    ob->flg &= ~(OBALLOC|OBREALLOC);
	}
    }
}

char *
x_strdup (s, file, line)
char *s;
char *file;
int line;
{
    char *ptr;

    if ((ptr = x_malloc (strlen (s) + 1, file, line)) != 0)
	strcpy (ptr, s);
    return ptr;
}

char *
x_calloc (nitems, size, file, line)
int nitems;
int size;
char *file;
int line;
{
    char *ptr;
    
    if ((ptr = x_malloc (nitems * size, file, line)) != 0)
	bzero (ptr, nitems * size);
    return ptr;
}

void *
x_xmalloc (size, file, line)
int size;
char *file;
int line;
{
    char *ptr;

    if (size == 0)
	size = 1;
    ptr = x_malloc (size, file, line);
    if (ptr == 0) {
	char buf[1400];
	sprintf (buf, "malloc failed: file %s, line %d\n", file, line);
	SWRITE (debfd, buf);
	exit (1);
    }
    return ptr;
}

#else /* DEBUG_MALLOC */

void *
xmalloc (size, file, line)
int size;
char *file;
int line;
{
    char *ptr;

    if (size == 0)
	size = 1;
    if ((ptr = malloc (size)) == 0) {
	char buf[1400];
	sprintf (buf, "malloc failed: file %s, line %d\n", file, line);
	write (2, buf, strlen (buf));
	exit (1);
    }
    return ptr;
}

#if 0
#ifdef IMA_PMAX
char *
strdup (s)
char *s;
{
    int len = strlen (s) + 1;
    char *malloc ();
    char *ptr = malloc (len);

    if (ptr)
	memcpy (ptr, s, len);
    return ptr;
}
#endif
#endif

#endif /* DEBUG_MALLOC */
